@@ -5,25 +5,14 @@
 
 import logging
 from inspect import getmembers, isclass
+from random import randint
 from sys import modules
 from typing import Dict, Optional
 
-from .const import (
-    ATTR_ALIAS,
-    ATTR_CLASS,
-    ATTR_FAKED,
-    DISCOVER_ALL,
-    DISCOVER_PARAMS,
-    DISCOVER_SCHEMA,
-    DISCOVER_STATUS,
-)
-from .entities import Entity, discovery_filter
+from .const import ATTR_ALIAS, ATTR_CLASS, ATTR_FAKED, Discover
+from .entities import Entity, discover_decorator
 from .protocol import Command, Priority  # TODO: constants to const.py
-from .protocol.address import (  # TODO: all required?
-    NON_DEV_ADDR,
-    dev_id_to_hex,
-    id_to_address,
-)
+from .protocol.address import NON_DEV_ADDR, id_to_address
 from .protocol.command import FUNC, TIMEOUT
 from .protocol.const import (
     _000C_DEVICE,
@@ -33,8 +22,6 @@ from .protocol.const import (
     ATTR_TEMP,
     ATTR_WINDOW_OPEN,
     DEVICE_CLASS,
-    DEVICE_HAS_BATTERY,
-    DEVICE_TABLE,
     DEVICE_TYPES,
     DOMAIN_TYPE_MAP,
     NUL_DEVICE_ID,
@@ -62,6 +49,7 @@ from .protocol import (  # noqa: F401, isort: skip
     _0404,
     _0418,
     _042F,
+    _0B04,
     _1030,
     _1060,
     _1090,
@@ -114,7 +102,7 @@ _LOGGER = logging.getLogger(__name__)
 if DEV_MODE:
     _LOGGER.setLevel(logging.DEBUG)
 
-_DEV_TYPE_TO_CLASS = {  # TODO: removw
+_DEV_TYPE_TO_CLASS = {  # TODO: *remove*
     None: DEVICE_CLASS.GEN,  # a generic, promotable device
     "00": DEVICE_CLASS.TRV,
     "01": DEVICE_CLASS.CTL,
@@ -146,9 +134,9 @@ class DeviceBase(Entity):
     """The Device base class (good for a generic device)."""
 
     _class = None
-    _types = tuple()
+    _types = tuple()  # TODO: needed?
 
-    def __init__(self, gwy, dev_addr, ctl=None, domain_id=None) -> None:
+    def __init__(self, gwy, dev_addr, ctl=None, domain_id=None, **kwargs) -> None:
         _LOGGER.debug("Creating a Device: %s (%s)", dev_addr.id, self.__class__)
         super().__init__(gwy)
 
@@ -165,22 +153,12 @@ class DeviceBase(Entity):
         self._parent = None
 
         self.addr = dev_addr
-        self.hex_id = dev_id_to_hex(dev_addr.id)
-        self.type = dev_addr.type
+        self.type = dev_addr.type  # DEX  # TODO: remove this attr
 
         self.devices = []  # [self]
         self.device_by_id = {}  # {self.id: self}
         self._iz_controller = None
 
-        if self.type in DEVICE_TABLE:
-            self._has_battery = DEVICE_TABLE[self.addr.type].get("has_battery")
-            self._is_actuator = DEVICE_TABLE[self.addr.type].get("is_actuator")
-            self._is_sensor = DEVICE_TABLE[self.addr.type].get("is_sensor")
-        else:
-            self._has_battery = None
-            self._is_actuator = None
-            self._is_sensor = None
-
         self._alias = None
         self._faked = None
         if self.id in gwy._include:
@@ -197,26 +175,26 @@ class DeviceBase(Entity):
             return NotImplemented
         return self.id < other.id
 
-    @discovery_filter
-    def _discover(self, discover_flag=DISCOVER_ALL) -> None:
-        # sometimes, battery-powered devices would respond to an RQ (e.g. bind mode)
+    @discover_decorator
+    def _discover(self, discover_flag=Discover.ALL) -> None:
+        # sometimes, battery-powered devices will respond to an RQ (e.g. bind mode)
 
-        if discover_flag & DISCOVER_SCHEMA:
-            self._send_cmd(_1FC9, retries=3)  # rf_bind
+        if discover_flag & Discover.SCHEMA:
+            self._make_cmd(_1FC9, retries=3)  # rf_bind
 
-        if discover_flag & DISCOVER_STATUS:
-            self._send_cmd(_0016, retries=3)  # rf_check
+        if discover_flag & Discover.STATUS:
+            self._make_cmd(_0016, retries=3)  # rf_check
 
-    def _send_cmd(self, code, **kwargs) -> None:
-        dest = kwargs.pop("dest_addr", self.id)
-        payload = kwargs.pop("payload", "00")
-        super()._send_cmd(code, dest, payload, **kwargs)
+    def _make_cmd(self, code, payload="00", **kwargs) -> None:
+        super()._make_cmd(code, self.id, payload, **kwargs)
 
     def _set_ctl(self, ctl) -> None:  # self._ctl
         """Set the device's parent controller, after validating it."""
 
         if self._ctl is ctl:
             return
+        if self._is_controller:  # HACK
+            return
         if self._ctl is not None:
             raise CorruptStateError(f"{self} changed controller: {self._ctl} to {ctl}")
 
@@ -227,7 +205,9 @@ class DeviceBase(Entity):
         self._ctl = ctl
         ctl.device_by_id[self.id] = self
         ctl.devices.append(self)
+
         _LOGGER.debug("%s: controller now set to %s", self, ctl)
+        return ctl
 
     def _handle_msg(self, msg) -> None:
         assert msg.src is self, f"msg inappropriately routed to {self}"
@@ -239,7 +219,6 @@ class DeviceBase(Entity):
         if not self._iz_controller and msg.code in CODE_ONLY_FROM_CTL:
             if self._iz_controller is None:
                 _LOGGER.info(f"{msg._pkt} # IS_CONTROLLER (00): is TRUE")
-                self._iz_controller = msg
                 self._make_tcs_controller(msg)
             elif self._iz_controller is False:  # TODO: raise CorruptStateError
                 _LOGGER.error(f"{msg._pkt} # IS_CONTROLLER (01): was FALSE, now True")
@@ -248,7 +227,7 @@ class DeviceBase(Entity):
     def has_battery(self) -> Optional[bool]:  # 1060
         """Return True if a device is battery powered (excludes battery-backup)."""
 
-        return self.type in DEVICE_HAS_BATTERY or _1060 in self._msgz
+        return isinstance(self, BatteryState) or _1060 in self._msgz
 
     @property
     def _is_controller(self) -> Optional[bool]:
@@ -268,9 +247,13 @@ class DeviceBase(Entity):
             m.src == self for m in self._msgs.values() if not m._expired
         )  # TODO: needs addressing
 
-    def _make_tcs_controller(self, msg=None):  # CH/DHW
+    def _make_tcs_controller(self, msg=None, **kwargs):  # CH/DHW
         """Create a TCS, and attach it to this controller."""
+        from .systems import create_system  # HACK: needs sorting
+
         self._iz_controller = msg or True
+        if self.type in ("01", "12", "22", "23", "34") and self._evo is None:  # DEX
+            self._evo = create_system(self._gwy, self, **kwargs)
 
     @property
     def schema(self) -> dict:
@@ -285,10 +268,12 @@ class DeviceBase(Entity):
 
     @property
     def params(self):
+        """Return the configurable attributes of the device."""
         return {}
 
     @property
     def status(self):
+        """Return the state attributes of the device."""
         return {}
 
 
@@ -297,12 +282,11 @@ class DeviceInfo:  # 10E0
     RF_BIND = "rf_bind"
     DEVICE_INFO = "device_info"
 
-    @discovery_filter
-    def _discover(self, discover_flag=DISCOVER_ALL) -> None:
-        if discover_flag & DISCOVER_SCHEMA:
+    def _discover(self, discover_flag=Discover.ALL) -> None:
+        if discover_flag & Discover.SCHEMA:
             try:
-                if RP in RAMSES_DEVICES[self.type][_10E0]:
-                    self._send_cmd(_10E0, retries=3)
+                if RP in RAMSES_DEVICES[self.type][_10E0]:  # DEX (convert to e.g. BDR)
+                    self._make_cmd(_10E0, retries=3)
             except KeyError:
                 pass
 
@@ -314,7 +298,7 @@ class DeviceInfo:  # 10E0
     def schema(self) -> dict:
         result = super().schema
         # result.update({self.RF_BIND: self._msg_value(_1FC9)})
-        if _10E0 in self._msgs or _10E0 in RAMSES_DEVICES.get(self.type, []):
+        if _10E0 in self._msgs or _10E0 in RAMSES_DEVICES.get(self.type, []):  # DEX
             result.update({self.DEVICE_INFO: self.device_info})
         return result
 
@@ -328,7 +312,7 @@ class Device(DeviceInfo, DeviceBase):
     def _handle_msg(self, msg) -> None:
         super()._handle_msg(msg)
 
-        if type(self) is Device and self.type == "30":  # self.__class__ is Device
+        if type(self) is Device and self.type == "30":  # self.__class__ is Device, DEX
             # TODO: the RFG codes need checking
             if msg.code in (_31D9, _31DA, _31E0) and msg.verb in (I_, RP):
                 self.__class__ = FanDevice
@@ -388,7 +372,9 @@ class Device(DeviceInfo, DeviceBase):
         if hasattr(parent, "devices") and self not in parent.devices:
             parent.devices.append(self)
             parent.device_by_id[self.id] = self
+
         _LOGGER.debug("Device %s: parent now set to %s", self, parent)
+        return parent
 
     @property
     def zone(self) -> Optional[Entity]:  # should be: Optional[Zone]
@@ -404,19 +390,21 @@ class Actuator:  # 3EF0, 3EF1
     ACTUATOR_STATE = "actuator_state"
     MODULATION_LEVEL = "modulation_level"  # percentage (0.0-1.0)
 
-    @discovery_filter
-    def _discover(self, discover_flag=DISCOVER_ALL) -> None:
+    def _discover(self, discover_flag=Discover.ALL) -> None:
         super()._discover(discover_flag=discover_flag)
 
-        if discover_flag & DISCOVER_STATUS and not self._faked:
+        if discover_flag & Discover.STATUS and not self._faked:
             # NOTE: No need to send periodic RQ/3EF1s to an OTB, use RQ/3220/11s
-            self._send_cmd(_3EF1)  # NOTE: No RPs to RQ/3EF0
+            self._make_cmd(_3EF1)  # NOTE: No RPs to RQ/3EF0
 
     def _handle_msg(self, msg) -> None:
         super()._handle_msg(msg)
 
+        if isinstance(self, OtbGateway):
+            return
+
         if msg.code == _3EF0 and msg.verb == I_ and not self._faked:
-            self._send_cmd(_3EF1, priority=Priority.LOW, retries=1)
+            self._make_cmd(_3EF1, priority=Priority.LOW, retries=1)
 
     @property
     def actuator_cycle(self) -> Optional[dict]:  # 3EF1
@@ -549,7 +537,7 @@ class Fakeable:
                 dst_id=msg.src.id,
                 callback={FUNC: bind_confirm, TIMEOUT: 3},
             )
-            self._gwy.send_cmd(cmd)
+            self._send_cmd(cmd)
 
         # assert code in SUPPORTED_CODES, f"Binding: {code} is not supported"
         self._1fc9_state = "waiting"
@@ -559,7 +547,7 @@ class Fakeable:
         )
 
     def _bind_request(self, code, callback=None):
-        """Initate a bind handshake: send the 1st packet of the handshake."""
+        """Initiate a bind handshake: send the 1st packet of the handshake."""
 
         # Bind request: CTL set to listen, STA initiates handshake (note 3C09/2309)
         # 22:13:52.527 070  I --- 34:021943 --:------ 34:021943 1FC9 024 00-3C09-8855B7 00-30C9-8855B7 00-0008-8855B7 00-1FC9-8855B7
@@ -582,7 +570,7 @@ class Fakeable:
             self._1fc9_state == "confirm"
 
             cmd = Command.put_bind(I_, code, self.id, dst_id=msg.src.id)
-            self._gwy.send_cmd(cmd)
+            self._send_cmd(cmd)
 
             if callback:
                 callback(msg)
@@ -593,7 +581,7 @@ class Fakeable:
         cmd = Command.put_bind(
             I_, code, self.id, callback={FUNC: bind_confirm, TIMEOUT: 3}
         )
-        self._gwy.send_cmd(cmd)
+        self._send_cmd(cmd)
 
     @property
     def schema(self) -> dict:
@@ -635,7 +623,7 @@ class Weather(Fakeable):  # 0002 (fakeable)
         # cmd = Command.put_zone_temp(
         #     self._gwy.hgi.id if self == self._gwy.hgi._faked_thm else self.id, value
         # )
-        self._gwy.send_cmd(cmd)
+        self._send_cmd(cmd)
 
     @property
     def status(self) -> dict:
@@ -677,8 +665,8 @@ class Temperature(Fakeable):  # 30C9 (fakeable)
         if not self._faked:
             raise RuntimeError(f"Can't set value for {self} (Faking is not enabled)")
 
-        self._gwy.send_cmd(Command.put_sensor_temp(self.id, value))
-        # lf._gwy.send_cmd(Command.get_zone_temp(self._ctl.id, self.zone.idx))
+        self._send_cmd(Command.put_sensor_temp(self.id, value))
+        # lf._send_cmd(Command.get_zone_temp(self._ctl.id, self.zone.idx))
 
     @property
     def status(self) -> dict:
@@ -698,12 +686,14 @@ class RelayDemand(Fakeable):  # 0008 (fakeable)
         if kwargs.get(ATTR_FAKED) is True or _3EF0 in kwargs.get(ATTR_FAKED, []):
             self._make_fake()
 
-    @discovery_filter
-    def _discover(self, discover_flag=DISCOVER_ALL) -> None:
+    @discover_decorator
+    def _discover(self, discover_flag=Discover.ALL) -> None:
         super()._discover(discover_flag=discover_flag)
 
-        if discover_flag & DISCOVER_STATUS and not self._faked:
-            self._send_cmd(_0008)  # NOTE: No RPs to RQ/0009
+        if discover_flag & Discover.STATUS and not self._faked:
+            self._send_cmd(
+                Command.get_relay_demand(self.id), priority=Priority.LOW, retries=1
+            )
 
     def _handle_msg(self, msg) -> None:
         if msg.src.id == self.id:
@@ -727,7 +717,7 @@ class RelayDemand(Fakeable):  # 0008 (fakeable)
 
             cmd = Command.put_actuator_state(self.id, mod_level)
             qos = {"priority": Priority.HIGH, "retries": 3}
-            [self._gwy.send_cmd(cmd, **qos) for _ in range(1)]
+            [self._send_cmd(cmd, **qos) for _ in range(1)]
 
         elif msg.code == _0009:  # can only be I, from a controller
             pass
@@ -736,14 +726,15 @@ class RelayDemand(Fakeable):  # 0008 (fakeable)
             pass
 
         elif msg.code == _3EF0 and msg.verb == I_:  # NOT RP, TODO: why????
-            self._send_cmd(_0008, priority=Priority.LOW, retries=1)
+            cmd = Command.get_relay_demand(self.id)
+            self._send_cmd(cmd, priority=Priority.LOW, retries=1)
 
         elif msg.code == _3EF1 and msg.verb == RQ:  # NOTE: WIP
             mod_level = 1.0
 
             cmd = Command.put_actuator_cycle(self.id, msg.src.id, mod_level, 600, 600)
             qos = {"priority": Priority.HIGH, "retries": 3}
-            [self._gwy.send_cmd(cmd, **qos) for _ in range(1)]
+            [self._send_cmd(cmd, **qos) for _ in range(1)]
 
         else:
             raise
@@ -797,7 +788,7 @@ class HGInterface(DeviceBase):  # HGI (18:), was GWY
 
     def _set_ctl(self, ctl) -> None:  # self._ctl
         """Set the device's parent controller, after validating it."""
-        _LOGGER.debug("%s: can't (really) have a controller %s", self)
+        _LOGGER.debug("%s: can't (really) have a controller %s", self, ctl)
 
     def _proc_schema(self, schema) -> None:
         if schema.get("fake_bdr"):
@@ -809,8 +800,8 @@ class HGInterface(DeviceBase):  # HGI (18:), was GWY
         if schema.get("fake_thm"):
             self._fake_thm = self._gwy._get_device(self.id, class_="BDR", faked=True)
 
-    @discovery_filter
-    def _discover(self, discover_flag=DISCOVER_ALL) -> None:
+    @discover_decorator
+    def _discover(self, discover_flag=Discover.ALL) -> None:
         # of no value for a HGI80-compatible device
         return
 
@@ -924,21 +915,27 @@ class Controller(Device):  # CTL (01):
         self._domain_id = "FF"
         self._evo = None
 
-        self._iz_controller = True
+        self._make_tcs_controller()
 
-    def _handle_msg(self, msg) -> bool:
+    def _handle_msg(self, msg) -> None:
         super()._handle_msg(msg)
 
-        # Route any messages to their heating systems
+        if msg.code == _000C:
+            [
+                self._gwy._get_device(d, ctl_id=self._ctl.id)
+                for d in msg.payload["devices"]
+            ]
+
+        # Route any messages to their heating systems, TODO: create dev first?
         if self._evo:
             self._evo._handle_msg(msg)
 
-    # @discovery_filter
-    # def _discover(self, discover_flag=DISCOVER_ALL) -> None:
+    # @discover_decorator
+    # def _discover(self, discover_flag=Discover.ALL) -> None:
     #     super()._discover(discover_flag=discover_flag)
 
-    #     if discover_flag & DISCOVER_SCHEMA:
-    #         pass  # self._send_cmd(_0000, retries=3)
+    #     if discover_flag & Discover.SCHEMA:
+    #         pass  # self._make_cmd(_0000, retries=3)
 
 
 class Programmer(Controller):  # PRG (23):
@@ -971,32 +968,32 @@ class UfhController(Device):  # UFC (02):
 
         self._iz_controller = True
 
-    @discovery_filter
-    def _discover(self, discover_flag=DISCOVER_ALL) -> None:
+    @discover_decorator
+    def _discover(self, discover_flag=Discover.ALL) -> None:
         super()._discover(discover_flag=discover_flag)
 
         # TODO: UFC may RP to an RQ/0001
 
-        if discover_flag & DISCOVER_SCHEMA:
+        if discover_flag & Discover.SCHEMA:
             [  # 0005: shows which channels are active - ?no use? (see above)
-                self._send_cmd(_0005, payload=f"00{zone_type}")
+                self._make_cmd(_0005, payload=f"00{zone_type}")
                 for zone_type in ("09",)  # _0005_ZONE_TYPE, also ("00", "04", "0F")
                 # for zone_type in _0005_ZONE_TYPE
             ]
 
             [  # 000C: used to find evo zone for each configured channel
-                self._send_cmd(_000C, payload=f"{idx:02X}{_000C_DEVICE.UFH}")
+                self._make_cmd(_000C, payload=f"{idx:02X}{_000C_DEVICE.UFH}")
                 for idx in range(8)  # for each possible UFH channel/circuit
             ]
 
-        # if discover_flag & DISCOVER_STATUS:
+        # if discover_flag & Discover.STATUS:
         #     [  # 22C9: no answer
-        #         self._send_cmd(_22C9, payload=f"{payload}")
+        #         self._make_cmd(_22C9, payload=f"{payload}")
         #         for payload in ("00", "0000", "01", "0100")
         #     ]
 
         #     [  # 3150: no answer
-        #         self._send_cmd(_3150, payload=f"{zone_idx:02X}")
+        #         self._make_cmd(_3150, payload=f"{zone_idx:02X}")
         #         for zone_idx in range(8)
         #     ]
 
@@ -1008,6 +1005,13 @@ class UfhController(Device):  # UFC (02):
             if msg.payload["zone_id"] is not None:
                 self._circuits[msg.payload["ufh_idx"]] = msg
 
+            # if dev_ids := msg.payload["devices"]:
+            #     if ctl := self._set_ctl(self._gwy._get_device(dev_ids[0])):
+            #         if zone := ctl._evo.zone_by_idx.get(msg.payload["zone_id"]):
+            #             zone.devices.append(self)
+            #             zone.device_by_id[self.id] = self
+            #             _LOGGER.debug("Device %s: added to Zone: %s", self, zone)
+
         elif msg.code == _22C9:
             if isinstance(msg.payload, list):
                 self._setpoints = msg
@@ -1019,6 +1023,13 @@ class UfhController(Device):  # UFC (02):
                 self._heat_demands = msg
             elif "domain_id" in msg.payload:
                 self._heat_demand = msg
+            elif zone_idx := msg.payload.get("zone_idx"):
+                if (
+                    self._ctl
+                    and self._ctl._evo
+                    and (zone := self._ctl._evo.zone_by_idx.get(zone_idx))
+                ):
+                    zone._handle_msg(msg)
             # else:
             #     pass  # update the self._circuits[]
 
@@ -1030,6 +1041,10 @@ class UfhController(Device):  # UFC (02):
             k: {"zone_idx": m.payload["zone_id"]} for k, m in self._circuits.items()
         }
 
+    @property
+    def zones(self) -> Optional[Dict]:  # 000C
+        return {m.payload["zone_id"]: {"ufh_idx": k} for k, m in self._circuits.items()}
+
         # def fix(k):
         #     return "zone_idx" if k == "zone_id" else k
 
@@ -1104,8 +1119,8 @@ class DhwSensor(BatteryState, Device):  # DHW (07): 10A0, 1260
     def _handle_msg(self, msg) -> None:
         super()._handle_msg(msg)
 
-        if msg.code == _1260:
-            self._gwy.send_cmd(Command.get_dhw_temp(self._ctl.id))
+        if msg.code == _1260 and self._ctl:  # device can be instatiated with a CTL
+            self._send_cmd(Command.get_dhw_temp(self._ctl.id))
 
     @property
     def dhw_params(self) -> Optional[dict]:  # 10A0
@@ -1146,14 +1161,14 @@ class ExtSensor(Weather, Device):  # EXT: 17
         return f"{self.id} ({self._domain_id}): {self.temperature}"
 
 
-class OtbGateway(Actuator, HeatDemand, Device):  # OTB (10): 22D9, 3220
+class OtbGateway(Actuator, HeatDemand, Device):  # OTB (10): 3220 (22D9, others)
     """The OTB class, specifically an OpenTherm Bridge (R8810A Bridge)."""
 
     _class = DEVICE_CLASS.OTB
     _types = ("10",)
 
-    BOILER_SETPOINT = "boiler_setpoint"
-    OPENTHERM_STATUS = "opentherm_status"
+    # BOILER_SETPOINT = "boiler_setpoint"
+    # OPENTHERM_STATUS = "opentherm_status"
     # _STATE = super().MODULATION_LEVEL
 
     SCHEMA_MSG_IDS = (
@@ -1167,7 +1182,7 @@ class OtbGateway(Actuator, HeatDemand, Device):  # OTB (10): 22D9, 3220
     )
     PARAMS_MSG_IDS = (
         0x38,  # .56: "DHW Setpoint (°C) (Remote parameter 1)",             # see: 0x06
-        0x39,  # .57: "Max CH water Setpoint (°C) (Remote parameters 2)",   # see: 0x06
+        0x39,  # .57: "Max CH water Setpoint (°C) (Remote parameter 2)",    # see: 0x06
         # These are error codes...
         0x05,  # ..5: "Fault flags & OEM codes",
         0x73,  # 115: "OEM diagnostic code",
@@ -1185,6 +1200,7 @@ class OtbGateway(Actuator, HeatDemand, Device):  # OTB (10): 22D9, 3220
     )
     STATUS_MSG_IDS = (
         0x00,  # ..0: "Master/Slave status flags",                          # not native
+        0x01,  # ..1: "CH water temperature Setpoint (°C)",                 # also R/W
         0x11,  # .17: "Relative Modulation Level (%)",
         0x12,  # .18: "Water pressure in CH circuit (bar)",
         0x13,  # .19: "Water flow rate in DHW circuit. (L/min)",
@@ -1194,7 +1210,7 @@ class OtbGateway(Actuator, HeatDemand, Device):  # OTB (10): 22D9, 3220
         0x1C,  # .28: "Return water temperature (°C)",
     )
     WRITE_MSG_IDS = (  # Write-Data, some may also Read-Data (will need to check)
-        0x01,  # ..1: "CH water temperature Setpoint (°C)",
+        0x01,  # ..1:  -see above-
         0x02,  # ..2: "Master configuration",
         0x0E,  # .14: "Maximum relative modulation level setting (%)",  # c.f. 0x11
         0x10,  # .16: "Room Setpoint (°C)",     # tell slave the room setpoint?
@@ -1217,48 +1233,71 @@ class OtbGateway(Actuator, HeatDemand, Device):  # OTB (10): 22D9, 3220
     def __repr__(self) -> str:
         return f"{self.id} ({self._domain_id}): {self.modulation_level}"  # 3EF0
 
-    @discovery_filter
-    def _discover(self, discover_flag=DISCOVER_ALL) -> None:
+    @discover_decorator
+    def _discover(self, discover_flag=Discover.ALL) -> None:
         # see: https://www.opentherm.eu/request-details/?post_ids=2944
 
         super()._discover(discover_flag=discover_flag)
 
-        if discover_flag & DISCOVER_SCHEMA:
+        if discover_flag & Discover.SCHEMA:
             [
-                self._gwy.send_cmd(Command.get_opentherm_data(self.id, m))
+                self._send_cmd(Command.get_opentherm_data(self.id, m))
                 for m in self.SCHEMA_MSG_IDS  # From OT v2.2: version numbers
                 if self._supported_msg.get(m) is not False
                 and (not self._opentherm_msg.get(m) or self._opentherm_msg[m]._expired)
             ]
 
-        if discover_flag & DISCOVER_PARAMS:
+        if discover_flag & Discover.PARAMS:
             [
-                self._gwy.send_cmd(Command.get_opentherm_data(self.id, m))
+                self._send_cmd(Command.get_opentherm_data(self.id, m))
                 for m in self.PARAMS_MSG_IDS
                 if self._supported_msg.get(m) is not False
                 and (not self._opentherm_msg.get(m) or self._opentherm_msg[m]._expired)
             ]
 
-        if discover_flag & DISCOVER_STATUS:
-            self._gwy.send_cmd(Command(RQ, _22D9, "00", self.id))
+            for code in (
+                # "1FD0",
+                # "2400",
+                # "2401",
+                # "2410",
+                # "2420"
+                "0150",
+                "1098",
+                "10B0",
+                "3221",
+                "3223",
+            ):
+                self._send_cmd(Command(RQ, code, "00", self.id))
+
+        if discover_flag & Discover.STATUS:
+            self._send_cmd(Command(RQ, "12F0", "00", self.id))
             [
-                self._gwy.send_cmd(Command.get_opentherm_data(self.id, m, retries=0))
+                self._send_cmd(Command.get_opentherm_data(self.id, m, retries=0))
                 for m in self.STATUS_MSG_IDS
                 if self._supported_msg.get(m) is not False
                 and (not self._opentherm_msg.get(m) or self._opentherm_msg[m]._expired)
             ]
 
     def _handle_msg(self, msg) -> None:
+        if msg.code == _3220 and msg._pkt.payload[4:] in ("121980", "1347AB", "1B47AB"):
+            if msg.payload[MSG_ID] not in self._supported_msg:
+                self._supported_msg[msg.payload[MSG_ID]] = None
+
+            elif self._supported_msg[msg.payload[MSG_ID]] is None:
+                self._supported_msg[msg.payload[MSG_ID]] = False
+                _LOGGER.warning(
+                    f"{msg._pkt} << OpenTherm: deprecating msg_id "
+                    f"{msg.payload[MSG_ID]}: it appears unsupported",
+                )
+
+        if msg.code == _3220 and not self._supported_msg.get(msg.payload[MSG_ID]):
+            return
+
         super()._handle_msg(msg)
 
-        if msg.code == _1FD4:  # every 30s
-            if msg.payload["ticker"] % 6 in (0, 2):  # twice every 3 mins
-                self._discover(discover_flag=DISCOVER_STATUS)
-            elif msg.payload["ticker"] % 60 in (1, 3):  # effectively once every 30 mins
-                self._discover(discover_flag=DISCOVER_PARAMS)
-
-        elif msg.code == _3220:  # all are RP
+        if msg.code == _3220:
             self._supported_msg[msg.payload[MSG_ID]] = msg.payload[MSG_TYPE] not in (
+                "Data-Invalid",
                 "Unknown-DataId",
                 "-reserved-",
             )
@@ -1270,16 +1309,24 @@ class OtbGateway(Actuator, HeatDemand, Device):  # OTB (10): 22D9, 3220
             return
 
     @property
-    def modulation_level(self) -> Optional[float]:  # 3EF0/3EF1
-        # TODO: for OTB, deprecate in favour of RP/3220/11
-        return self._msg_value((_3EF0, _3EF1), key=self.MODULATION_LEVEL)
-
-    @property
-    def boiler_water_temperature(self) -> Optional[float]:  # 3220/19
+    def boiler_output_temp(self) -> Optional[float]:  # 3220/19
         return self._ot_msg_value(0x19)
 
     @property
-    def ch_water_pressure(self) -> Optional[float]:  # 3220/12
+    def boiler_return_temp(self) -> Optional[float]:  # 3220/1C
+        return self._ot_msg_value(0x1C)
+
+    @property
+    def boiler_setpoint(self) -> Optional[float]:  # 3220/01 (22D9)
+        # return self._msg_value(_22D9, key=self.BOILER_SETPOINT)
+        return self._ot_msg_value(0x01)
+
+    @property
+    def ch_max_setpoint(self) -> Optional[float]:  # 3220/39 (1081)
+        return self._ot_msg_value(0x39)
+
+    @property
+    def ch_water_pressure(self) -> Optional[float]:  # 3220/12 (1300)
         return self._ot_msg_value(0x12)
 
     @property
@@ -1287,21 +1334,22 @@ class OtbGateway(Actuator, HeatDemand, Device):  # OTB (10): 22D9, 3220
         return self._ot_msg_value(0x13)
 
     @property
-    def dhw_temperature(self) -> Optional[float]:  # 3220/1A
+    def dhw_setpoint(self) -> Optional[float]:  # 3220/38 (10A0)
+        return self._ot_msg_value(0x89)
+
+    @property
+    def dhw_temp(self) -> Optional[float]:  # 3220/1A (1260)
         return self._ot_msg_value(0x1A)
 
     @property
-    def relative_modulation_level(self) -> Optional[float]:  # 3220/11
+    def outside_temp(self) -> Optional[float]:  # 3220/1B (1290)
+        return self._ot_msg_value(0x1B)
+
+    @property
+    def rel_modulation_level(self) -> Optional[float]:  # 3220/11 (3EFx)
+        # return self._msg_value((_3EF0, _3EF1), key=self.MODULATION_LEVEL)
         return self._ot_msg_value(0x11)
 
-    @property
-    def return_water_temperature(self) -> Optional[float]:  # 3220/1C
-        return self._ot_msg_value(0x1C)
-
-    @property
-    def boiler_setpoint(self) -> Optional[float]:  # 22D9
-        return self._msg_value(_22D9, key=self.BOILER_SETPOINT)
-
     @staticmethod
     def _msg_name(msg) -> str:
         return (
@@ -1336,24 +1384,17 @@ class OtbGateway(Actuator, HeatDemand, Device):  # OTB (10): 22D9, 3220
 
     @property
     def opentherm_status(self) -> dict:
-        opentherm_status = {
-            "boiler_water_temperature": self.boiler_water_temperature,
+        return {
+            "boiler_output_temp": self.boiler_output_temp,
+            "boiler_return_temp": self.boiler_return_temp,
+            "boiler_setpoint": self.boiler_setpoint,
+            "ch_max_setpoint": self.ch_max_setpoint,
             "ch_water_pressure": self.ch_water_pressure,
             "dhw_flow_rate": self.dhw_flow_rate,
-            "dhw_temperature": self.dhw_temperature,
-            "relative_modulation_level": self.relative_modulation_level,
-            "return_water_temperature": self.return_water_temperature,
-        }
-        others = {
-            self._msg_name(v): {
-                x: y for x, y in v.payload.items() if x.startswith(VALUE)
-            }
-            for k, v in self._opentherm_msg.items()
-            if self._supported_msg.get(int(k, 16)) and k in ("00", "1B")
-        }
-        return {
-            **opentherm_status,
-            "other_state_attrs": others,
+            "dhw_setpoint": self.dhw_setpoint,
+            "dhw_temp": self.dhw_temp,
+            "outside_temp": self.outside_temp,
+            "rel_modulation_level": self.rel_modulation_level,
         }
         # return {
         #     slugify(self._opentherm_msg[msg_id].payload[MSG_NAME]): (
@@ -1367,7 +1408,6 @@ class OtbGateway(Actuator, HeatDemand, Device):  # OTB (10): 22D9, 3220
     def schema(self) -> dict:
         return {
             **super().schema,
-            #  grep '47AB ' | grep -vE '(13|1A|1B)47AB'
             "known_msg_ids": {
                 f"{k:02X}": OPENTHERM_MESSAGES[k].get("var", f"{k:02X}")
                 if k in OPENTHERM_MESSAGES
@@ -1376,6 +1416,7 @@ class OtbGateway(Actuator, HeatDemand, Device):  # OTB (10): 22D9, 3220
                 if v
             },
             "opentherm_schema": self.opentherm_schema,
+            # "supported_msg": self._supported_msg,
         }
 
     @property
@@ -1391,9 +1432,7 @@ class OtbGateway(Actuator, HeatDemand, Device):  # OTB (10): 22D9, 3220
         # bruce:     [0, 3, 5,    12, 13, 17, 18, 25, 27, 28, 48, 49, 56, 125]
         return {
             **super().status,
-            self.BOILER_SETPOINT: self.boiler_setpoint,
-            self.OPENTHERM_STATUS: self.opentherm_status,
-            self.MODULATION_LEVEL: self.modulation_level,
+            "opentherm_status": self.opentherm_status,
         }
 
 
@@ -1478,9 +1517,11 @@ class BdrSwitch(Actuator, RelayDemand, Device):  # BDR (13):
     def __repr__(self) -> str:
         return f"{self.id} ({self._domain_id}): {self.relay_demand}"
 
-    @discovery_filter
-    def _discover(self, discover_flag=DISCOVER_ALL) -> None:
-        """The BDRs have one of six roles:
+    @discover_decorator
+    def _discover(self, discover_flag=Discover.ALL) -> None:
+        """Discover BDRs.
+
+        The BDRs have one of six roles:
          - heater relay *or* a heat pump relay (alternative to an OTB)
          - DHW hot water valve *or* DHW heating valve
          - Zones: Electric relay *or* Zone valve relay
@@ -1495,8 +1536,8 @@ class BdrSwitch(Actuator, RelayDemand, Device):  # BDR (13):
 
         super()._discover(discover_flag=discover_flag)
 
-        if discover_flag & DISCOVER_PARAMS and not self._faked:
-            self._send_cmd(_1100)
+        if discover_flag & Discover.PARAMS and not self._faked:
+            self._send_cmd(Command.get_tpi_params(self.id))  # or: self._ctl.id
 
     def _handle_msg(self, msg) -> None:
         super()._handle_msg(msg)
@@ -1507,7 +1548,7 @@ class BdrSwitch(Actuator, RelayDemand, Device):  # BDR (13):
         # elif msg.code == _3B00 and msg.verb == I_:
         #     pass  # only a heater_relay will I/3B00
         #     # for code in (_0008, _3EF1):
-        #     #     self._send_cmd(code, delay=1)
+        #     #     self._make_cmd(code, delay=1)
 
     @property
     def active(self) -> Optional[bool]:  # 3EF0, 3EF1
@@ -1580,12 +1621,16 @@ class TrvActuator(BatteryState, HeatDemand, Setpoint, Temperature, Device):  # T
         }
 
 
-class FanSwitch(BatteryState, Device):  # SWI (39):
+class FanSwitch(BatteryState, Device):  # SWI (39): I/22F[13]
     """The FAN (switch) class, such as a 4-way switch.
 
     The cardinal codes are 22F1, 22F3.
     """
 
+    # every /15
+    # RQ --- 32:166025 30:079129 --:------ 31DA 001 21
+    # RP --- 30:079129 32:166025 --:------ 31DA 029 21EF00026036EF7FFF7FFF7FFF7FFF0002EF18FFFF000000EF7FFF7FFF
+
     _class = DEVICE_CLASS.SWI
     _types = ("39",)
 
@@ -1620,12 +1665,19 @@ class FanSwitch(BatteryState, Device):  # SWI (39):
         }
 
 
-class FanDevice(Device):  # FAN (20/37): I/31D[9A]
+class FanDevice(Device):  # FAN (20/37): RP/31DA, I/31D[9A]
     """The Ventilation class.
 
-    The cardinal code are 31D9, 31DA.
+    The cardinal code are 31D9, 31DA.  Signature is RP/31DA.
     """
 
+    # Itho Daalderop (NL)
+    # Heatrae Sadia (UK)
+    # Nuaire (UK), e.g. DRI-ECO-PIV
+
+    # every /30
+    # 30:079129 --:------ 30:079129 31D9 017 2100FF0000000000000000000000000000
+
     _class = DEVICE_CLASS.FAN
     _types = ("20", "37")
 
@@ -1667,7 +1719,7 @@ class FanDevice(Device):  # FAN (20/37): I/31D[9A]
         }
 
 
-class FanSensorHumidity(BatteryState, Device):  # HUM (32) Humidity sensor:
+class FanSensorHumidity(BatteryState, Device):  # HUM (32) I/12(98|A0)
     """The Sensor class for a humidity sensor.
 
     The cardinal code is 12A0.
@@ -1719,30 +1771,40 @@ DEVICE_BY_ID_TYPE = {
 }  # e.g. "01": Controller,
 
 
+DEVICE_KLASS_BY_SIGNATURE = {
+    "FAN": ((RP, _31DA), (I_, _31D9), (I_, _31DA)),
+    "SWI": ((I_, _22F1), (I_, _22F3)),
+    "HUM": ((I_, _12A0)),
+    "C02": ((I_, _1298)),
+}
+
+
 def create_device(gwy, dev_id, dev_class=None, **kwargs) -> Device:
     """Create a device, and optionally perform discovery & start polling."""
 
     dev_addr = id_to_address(dev_id)
 
     if dev_class is None:
-        if dev_addr.type != "30":  # could be RFG or VNT
-            dev_class = _DEV_TYPE_TO_CLASS.get(dev_addr.type, DEVICE_CLASS.GEN)
+        if dev_addr.type != "30":  # could be RFG or VNT  # DEX
+            dev_class = _DEV_TYPE_TO_CLASS.get(dev_addr.type, DEVICE_CLASS.GEN)  # DEX
         else:
             dev_class = DEVICE_CLASS.GEN  # generic
 
     device = DEVICE_BY_CLASS.get(dev_class, Device)(gwy, dev_addr, **kwargs)
 
-    gwy._add_task(
-        device._discover, discover_flag=DISCOVER_SCHEMA, delay=0, period=86400
-    )
-
-    if dev_class == DEVICE_CLASS.OTB:
+    if not gwy.serial_port:
         return device
 
     gwy._add_task(
-        device._discover, discover_flag=DISCOVER_PARAMS, delay=0, period=21600
+        device._discover, discover_flag=Discover.SCHEMA, delay=0, period=86400
     )
 
-    gwy._add_task(device._discover, discover_flag=DISCOVER_STATUS, delay=0, period=900)
+    delay = randint(10, 20)
+    gwy._add_task(
+        device._discover, discover_flag=Discover.PARAMS, delay=delay, period=21600
+    )  # 21600 = 60*60*6
+    gwy._add_task(
+        device._discover, discover_flag=Discover.STATUS, delay=delay + 3, period=60
+    )
 
     return device
