@@ -15,12 +15,14 @@ from datetime import timedelta as td
 from types import SimpleNamespace
 from typing import Any, Optional, Tuple, Union
 
-from .address import HGI_DEV_ADDR, NON_DEV_ADDR, NUL_DEV_ADDR, dev_id_to_hex, pkt_addrs
+from .address import HGI_DEV_ADDR, NON_DEV_ADDR, NUL_DEV_ADDR, Address, pkt_addrs
 from .const import COMMAND_REGEX, SYSTEM_MODE, ZONE_MODE
 from .exceptions import ExpiredCallbackError, InvalidPacketError
 from .frame import PacketBase, pkt_header
-from .helpers import dt_now, dtm_to_hex, dts_to_hex, str_to_hex, temp_to_hex
+from .helpers import dt_now, dtm_to_hex, str_to_hex, temp_to_hex, timestamp
 from .opentherm import parity
+from .parsers import LOOKUP_PUZZ
+from .version import VERSION
 
 from .const import I_, RP, RQ, W_, __dev_mode__  # noqa: F401, isort: skip
 from .const import (  # noqa: F401, isort: skip
@@ -148,10 +150,21 @@ def validate_command(has_zone=None):
         ) as exc:
             _LOGGER.exception(f"validate_command(): {exc}")
 
+    def validate_zone_idx(zone_idx) -> int:
+        if isinstance(zone_idx, str):
+            zone_idx = "FA" if zone_idx == "HW" else zone_idx
+        zone_idx = zone_idx if isinstance(zone_idx, int) else int(zone_idx, 16)
+        if 0 > zone_idx > 15 and zone_idx != 0xFA:
+            raise ValueError("Invalid value for zone_idx")
+        return zone_idx
+
     def device_decorator(fcn):
         @functools.wraps(fcn)
         def wrapper(cls, dst_id, *args, **kwargs) -> Any:
 
+            if "zone_idx" in kwargs:
+                kwargs["zone_idx"] = validate_zone_idx(kwargs["zone_idx"])
+
             return _wrapper(fcn, cls, dst_id, *args, **kwargs)
 
         return wrapper
@@ -160,11 +173,7 @@ def validate_command(has_zone=None):
         @functools.wraps(fcn)
         def wrapper(cls, ctl_id, zone_idx, *args, **kwargs) -> Any:
 
-            if isinstance(zone_idx, str):
-                zone_idx = "FA" if zone_idx == "HW" else zone_idx
-            zone_idx = zone_idx if isinstance(zone_idx, int) else int(zone_idx, 16)
-            if 0 > zone_idx > 15 and zone_idx != 0xFA:
-                raise ValueError("Invalid value for zone_idx")
+            zone_idx = validate_zone_idx(zone_idx)
 
             return _wrapper(fcn, cls, ctl_id, zone_idx, *args, **kwargs)
 
@@ -342,7 +351,7 @@ class Command(PacketBase):
         )
 
         if not COMMAND_REGEX.match(self._frame[4:]):
-            raise InvalidPacketError("Invalid packet structure")
+            raise InvalidPacketError(f"{self._frame[4:]} < Invalid packet structure")
 
     @staticmethod
     def _is_valid_operand(other) -> bool:
@@ -487,6 +496,12 @@ class Command(PacketBase):
         payload = f"0080{msg_id:02X}0000" if parity(msg_id) else f"0000{msg_id:02X}0000"
         return cls(RQ, _3220, payload, dev_id, **kwargs)
 
+    @classmethod  # constructor for RQ/0008
+    @validate_command()  # has_zone=Optional
+    def get_relay_demand(cls, dev_id: str, zone_idx: Union[int, str] = "00", **kwargs):
+        """Constructor to get the demand of a relay/zone (c.f. parser_0008)."""
+        return cls(RQ, _0008, zone_idx, dev_id, **kwargs)
+
     @classmethod  # constructor for RQ/0404
     @validate_command(has_zone=True)
     def get_schedule_fragment(
@@ -502,8 +517,10 @@ class Command(PacketBase):
         Usually a zone, but will be the DHW schedule if zone_idx == 0xFA, 'FA', or 'HW'.
         """
 
-        header = "0023000800" if zone_idx == 0xFA else f"{zone_idx:02X}20000800"
-        payload = f"{header}{frag_idx + 1:02X}{frag_cnt:02X}"
+        header = "00230008" if zone_idx == 0xFA else f"{zone_idx:02X}200008"
+        frag_length = 0
+
+        payload = f"{header}{frag_length:02X}{frag_idx + 1:02X}{frag_cnt:02X}"
         return cls(RQ, _0404, payload, ctl_id, **kwargs)
 
     @classmethod  # constructor for RQ/0100
@@ -573,9 +590,11 @@ class Command(PacketBase):
 
     @classmethod  # constructor for RQ/1100
     @validate_command()
-    def get_tpi_params(cls, ctl_id: str, **kwargs):
+    def get_tpi_params(cls, dev_id: str, domain_id=None, **kwargs):
         """Constructor to get the TPI params of a system (c.f. parser_1100)."""
-        return cls(RQ, _1100, "FC", ctl_id, **kwargs)
+        if domain_id is None:
+            domain_id = "00" if dev_id[:2] == "13" else "FC"
+        return cls(RQ, _1100, domain_id, dev_id, **kwargs)
 
     @classmethod  # constructor for W/1100
     @validate_command()
@@ -791,7 +810,8 @@ class Command(PacketBase):
         """
         # RP --- 13:049798 18:006402 --:------ 3EF1 007 00-0126-0126-00-FF
 
-        # assert dev_id[:2] == "13"
+        assert src_id[:2] == "13"  # dex
+
         payload = f"00{actuator_countdown:04X}"
         payload += f"{cycle_countdown:04X}" if cycle_countdown is not None else "7FFF"
         payload += f"{int(mod_level * 200):02X}FF"
@@ -807,7 +827,8 @@ class Command(PacketBase):
         #  I --- 13:049798 --:------ 13:049798 3EF0 003 00C8FF
         #  I --- 13:106039 --:------ 13:106039 3EF0 003 0000FF
 
-        # assert dev_id[:2] == "13"
+        assert dev_id[:2] == "13"  # dex
+
         payload = "007FFF" if mod_level is None else f"00{int(mod_level * 200):02X}FF"
         return cls.packet(I_, _3EF0, payload, addr0=dev_id, addr2=dev_id, **kwargs)
 
@@ -819,7 +840,7 @@ class Command(PacketBase):
         #  W --- 01:145038 34:021943 --:------ 1FC9 006 00-2309-06368E
         #  I --- 34:021943 01:145038 --:------ 1FC9 006 00-2309-8855B7
 
-        hex_id = dev_id_to_hex(src_id)
+        hex_id = Address.convert_to_hex(src_id)
         codes = (list(codes) if isinstance(codes, tuple) else [codes]) + [_1FC9]
 
         if dst_id is None and verb == I_:
@@ -846,7 +867,8 @@ class Command(PacketBase):
         This is for use by a faked HB85 or similar.
         """
 
-        # assert dev_id[:2] == "17"
+        assert dev_id[:2] == "17"  # dex
+
         payload = f"00{temp_to_hex(temperature)}01"
         return cls.packet(I_, _0002, payload, addr0=dev_id, addr2=dev_id, **kwargs)
 
@@ -859,7 +881,8 @@ class Command(PacketBase):
         """
         #  I --- 34:021943 --:------ 34:021943 30C9 003 000C0D
 
-        # assert dev_id[:2] in ("03", "12", "22", "34")
+        assert dev_id[:2] in ("03", "12", "22", "34")  # dex
+
         payload = f"00{temp_to_hex(temperature)}"
         return cls.packet(I_, _30C9, payload, addr0=dev_id, addr2=dev_id, **kwargs)
 
@@ -879,30 +902,36 @@ class Command(PacketBase):
         Usually a zone, but will be the DHW schedule if zone_idx == 0xFA, 'FA', or 'HW'..
         """
 
-        header = "0023000800" if zone_idx == 0xFA else f"{zone_idx:02X}20000800"
+        header = "00230008" if zone_idx == 0xFA else f"{zone_idx:02X}200008"
         frag_length = int(len(fragment) / 2)
 
-        payload = f"{header}{frag_length}{frag_idx + 1:02X}{frag_cnt:02X}"
+        payload = f"{header}{frag_length:02X}{frag_idx + 1:02X}{frag_cnt:02X}{fragment}"
         return cls(W_, _0404, payload, ctl_id, **kwargs)
 
     @classmethod  # constructor for internal use only
-    def _puzzle(
-        cls, msg_type="01", message=None, ordinal=0, interval=0, length=None, **kwargs
-    ):
+    def _puzzle(cls, msg_type=None, message="", **kwargs):
 
-        if msg_type == "00":
-            payload = f"0000{dts_to_hex(dt_now())}7F"
-            payload += f"{str_to_hex(message)}7F"
+        if msg_type is None:
+            msg_type = "12" if message else "10"
 
-        elif msg_type in ("01", "02", "03"):
-            payload = f"00{msg_type}{str_to_hex(message)}7F"
+        assert msg_type in LOOKUP_PUZZ, "Invalid/deprecated Puzzle type"
 
+        kwargs["priority"] = kwargs.pop("priority", Priority.HIGHEST)
+        if msg_type == "10":
+            kwargs["disable_backoff"] = kwargs.pop("disable_backoff", True)
+            kwargs["retries"] = kwargs.pop("retries", 24)
+
+        payload = f"00{msg_type}"
+
+        if msg_type != "13":
+            payload += f"{int(timestamp() * 1000):012X}"
+
+        if msg_type == "10":
+            payload += str_to_hex(f"v{VERSION}")
+        elif msg_type == "11":
+            payload += str_to_hex(message[:4] + message[5:7] + message[8:])
         else:
-            payload = f"007F{dts_to_hex(dt_now())}7F"
-            payload += f"{ordinal % 0x10000:04X}7F{int(interval * 100):04X}7F"
-
-        if length:
-            payload = payload.ljust(length * 2, "F")
+            payload += str_to_hex(message)
 
         return cls(I_, _PUZZ, payload[:48], NUL_DEV_ADDR.id, **kwargs)
 
@@ -912,6 +941,7 @@ _COMMANDS = {
     f"{I_}/{_0002}": Command.put_outdoor_temp,
     f"{RQ}/{_0004}": Command.get_zone_name,
     f"{W_}/{_0004}": Command.set_zone_name,
+    f"{RQ}/{_0008}": Command.get_relay_demand,
     f"{RQ}/{_000A}": Command.get_zone_config,
     f"{W_}/{_000A}": Command.set_zone_config,
     f"{RQ}/{_0100}": Command.get_system_language,
@@ -953,52 +983,51 @@ class FaultLog:  # 0418  # TODO: used a NamedTuple
         # self._evo = ctl._evo
         self._gwy = ctl._gwy
 
-        self._fault_log = None
-        self._fault_log_done = None
+        self._faultlog = None
+        self._faultlog_done = None
 
         self._START = 0x00  # max 0x3E
         self._limit = 0x06
 
     def __repr_(self) -> str:
-        return json.dumps(self._fault_log) if self._fault_log_done else None
+        return json.dumps(self._faultlog) if self._faultlog_done else None
 
     def __str_(self) -> str:
         return f"{self._ctl} (fault log)"
 
-    @property
-    def fault_log(self) -> Optional[dict]:
-        """Return the fault log of a system."""
-        if not self._fault_log_done:
-            return
+    @staticmethod
+    def _is_valid_operand(other) -> bool:
+        return hasattr(other, "verb") and hasattr(other, "_pkt")
 
-        result = {
-            x: {k: v for k, v in y.items() if k[:1] != "_"}
-            for x, y in self._fault_log.items()
-        }
+    def __eq__(self, other) -> bool:
+        if not self._is_valid_operand(other):
+            return NotImplemented
+        return (self.verb, self._pkt.payload) == (other.verb, other.self._pkt.payload)
 
-        return {k: [x for x in v.values()] for k, v in result.items()}
-
-    async def get_fault_log(
+    async def get_faultlog(
         self, start=0, limit=6, force_refresh=None
     ) -> Optional[dict]:
         """Get the fault log of a system."""
-        _LOGGER.debug("FaultLog(%s).get_fault_log()", self)
+        _LOGGER.debug("FaultLog(%s).get_faultlog()", self)
+
+        if self._gwy.config.disable_sending:
+            raise RuntimeError("Sending is disabled")
 
         self._START = 0 if start is None else start
         self._limit = 6 if limit is None else limit
 
-        self._fault_log = {}  # TODO: = namedtuple("Fault", "timestamp fault_state ...")
-        self._fault_log_done = None
+        self._faultlog = {}  # TODO: = namedtuple("Fault", "timestamp fault_state ...")
+        self._faultlog_done = None
 
         self._rq_log_entry(log_idx=self._START)  # calls loop.create_task()
 
         time_start = dt.now()
-        while not self._fault_log_done:
+        while not self._faultlog_done:
             await asyncio.sleep(TIMER_SHORT_SLEEP)
             if dt.now() > time_start + TIMER_LONG_TIMEOUT * 2:
                 raise ExpiredCallbackError("failed to obtain log entry (long)")
 
-        return self.fault_log
+        return self.faultlog
 
     def _rq_log_entry(self, log_idx=0):
         """Request the next log entry."""
@@ -1008,7 +1037,7 @@ class FaultLog:  # 0418  # TODO: used a NamedTuple
             _LOGGER.debug("FaultLog(%s)._proc_log_entry(%s)", self.id, msg)
 
             if not msg:
-                self._fault_log_done = True
+                self._faultlog_done = True
                 # raise ExpiredCallbackError("failed to obtain log entry (short)")
                 return
 
@@ -1016,14 +1045,14 @@ class FaultLog:  # 0418  # TODO: used a NamedTuple
             log_idx = int(log.pop("log_idx", "00"), 16)
             if not log:  # null response (no payload)
                 # TODO: delete other callbacks rather than waiting for them to expire
-                self._fault_log_done = True
+                self._faultlog_done = True
                 return
 
-            self._fault_log[log_idx] = log  # TODO: make a named tuple
+            self._faultlog[log_idx] = log  # TODO: make a named tuple
             if log_idx < self._limit:
                 self._rq_log_entry(log_idx + 1)
             else:
-                self._fault_log_done = True
+                self._faultlog_done = True
 
         # TODO: (make method) register callback for null response (no payload)
         null_header = "|".join((RP, self.id, _0418))
@@ -1037,3 +1066,20 @@ class FaultLog:  # 0418  # TODO: used a NamedTuple
         self._gwy.send_cmd(
             Command.get_system_log_entry(self._ctl.id, log_idx, callback=rq_callback)
         )
+
+    @property
+    def faultlog(self) -> Optional[dict]:
+        """Return the fault log of a system."""
+        if not self._faultlog_done:
+            return
+
+        result = {
+            x: {k: v for k, v in y.items() if k[:1] != "_"}
+            for x, y in self._faultlog.items()
+        }
+
+        return {k: [x for x in v.values()] for k, v in result.items()}
+
+    @property
+    def _faultlog_outdated(self) -> bool:
+        return self._faultlog_done and len(self._faultlog_done) and self._faultlog_done
