@@ -15,7 +15,7 @@ from typing import Callable, List, Optional, Tuple
 
 from .command import ARGS, DEAMON, EXPIRES, FUNC, TIMEOUT, Command
 from .const import __dev_mode__
-from .exceptions import CorruptStateError
+from .exceptions import CorruptStateError, InvalidPacketError
 from .message import Message
 
 DONT_CREATE_MESSAGES = 3  # duplicate
@@ -23,7 +23,7 @@ DONT_CREATE_MESSAGES = 3  # duplicate
 DEV_MODE = __dev_mode__ and False
 
 _LOGGER = logging.getLogger(__name__)
-# _LOGGER.setLevel(logging.INFO)  # TODO: needs fixing - this should be the default
+# _LOGGER.setLevel(logging.WARNING)
 if DEV_MODE:
     _LOGGER.setLevel(logging.DEBUG)
 
@@ -131,7 +131,6 @@ class MessageTransport(asyncio.Transport):
                 #     continue
 
                 self._que.task_done()
-                # if self._write_buffer_paused:
                 self.get_write_buffer_size()
 
             _LOGGER.error("MsgTransport.pkt_dispatcher(): connection_lost(None)")
@@ -178,7 +177,10 @@ class MessageTransport(asyncio.Transport):
         if self._gwy.config.reduce_processing >= DONT_CREATE_MESSAGES:
             return
 
-        msg = Message(self._gwy, pkt)  # trap/logs all invalid msgs appropriately
+        try:
+            msg = Message(self._gwy, pkt)  # trap/logs all invalid msgs appropriately
+        except InvalidPacketError:
+            return
 
         # NOTE: msg._pkt._hdr is expensive - don't call it unless there's callbacks
         if self._callbacks and msg._pkt._hdr in self._callbacks:
@@ -253,7 +255,7 @@ class MessageTransport(asyncio.Transport):
         return self._protocols
 
     def is_reading(self) -> Optional[bool]:
-        """Return True if the transport is receiving."""
+        """Return True if the transport is receiving new data."""
         _LOGGER.debug("MsgTransport.is_reading()")
 
         raise NotImplementedError
@@ -278,6 +280,40 @@ class MessageTransport(asyncio.Transport):
 
         raise NotImplementedError
 
+    def _clear_write_buffer(self):
+        """Empty the dispatch queue."""
+
+        self._pause_protocols()
+        while not self._que.empty():
+            try:
+                self._que.get_nowait()
+            except Empty:
+                continue
+            self._que.task_done()
+        self.get_write_buffer_size()
+
+    def _pause_protocols(self, force=None):
+        """Pause the other end."""
+
+        if not self._write_buffer_paused or force:
+            self._write_buffer_paused = True
+            [p.pause_writing() for p in self._protocols]
+
+    def _resume_protocols(self, force=None):
+        """Resume the other end."""
+
+        if self._write_buffer_paused or force:
+            self._write_buffer_paused = False
+            [p.resume_writing() for p in self._protocols]
+
+    def get_write_buffer_limits(self) -> Tuple[int, int]:
+        """Get the high and low watermarks for write flow control.
+
+        Return a tuple (low, high) where low and high are positive number of bytes.
+        """
+
+        return self._write_buffer_limit_low, self._write_buffer_limit_high
+
     def set_write_buffer_limits(self, high=None, low=None):
         """Set the high- and low-water limits for write flow control.
 
@@ -294,29 +330,30 @@ class MessageTransport(asyncio.Transport):
         """
         _LOGGER.debug("MsgTransport.set_write_buffer_limits()")
 
-        self._write_buffer_limit_high = 10 if high is None else high
-        self._write_buffer_limit_low = (
-            int(self._write_buffer_limit_high * 0.8) if low is None else low
+        self._write_buffer_limit_high = int(
+            self.MAX_BUFFER_SIZE
+            if high is None
+            else max((min((high, self.MAX_BUFFER_SIZE)), 0))
+        )
+        self._write_buffer_limit_low = int(
+            self._write_buffer_limit_high * 0.8
+            if low is None
+            else min((max((low, 0)), high))
         )
-
-        assert 0 <= self._write_buffer_limit_low <= self._write_buffer_limit_high
 
         self.get_write_buffer_size()
 
-    def get_write_buffer_size(self):
+    def get_write_buffer_size(self) -> int:
         """Return the current size of the write buffer."""
         _LOGGER.debug("MsgTransport.get_write_buffer_size()")
 
         qsize = self._que.qsize()
 
-        if not self._write_buffer_paused:
-            if qsize >= self._write_buffer_limit_high:
-                self._write_buffer_paused = True
-                [p.pause_writing() for p in self._protocols]
+        if qsize >= self._write_buffer_limit_high:
+            self._pause_protocols()
 
-        elif qsize <= self._write_buffer_limit_high:
-            self._write_buffer_paused = False
-            [p.resume_writing() for p in self._protocols]
+        elif qsize <= self._write_buffer_limit_low:
+            self._resume_protocols()
 
         return qsize
 
@@ -331,9 +368,11 @@ class MessageTransport(asyncio.Transport):
         if self._is_closing:
             raise RuntimeError("MsgTransport is closing or has closed")
 
+        if self._write_buffer_paused:
+            raise RuntimeError("MsgTransport: write buffer is paused")
+
         if self._gwy.config.disable_sending:
-            message = "MsgTransport.write(%s): sending disabled: discarded"
-            (_LOGGER.info if DEV_MODE else _LOGGER.debug)(message, cmd)
+            raise RuntimeError("MsgTransport: sending is disabled (cmd discarded)")
 
         else:
             if not self._dispatcher:  # TODO: do better?
@@ -344,7 +383,7 @@ class MessageTransport(asyncio.Transport):
             except Full:
                 pass  # TODO: why? - consider restarting the dispatcher
 
-        # self.get_write_buffer_size()  # TODO: how to auto unpause?
+        self.get_write_buffer_size()
 
     def writelines(self, list_of_cmds):
         """Write a list (or any iterable) of data bytes to the transport.
@@ -401,7 +440,7 @@ class MessageProtocol(asyncio.Protocol):
 
     def __init__(self, gwy, callback: Callable) -> None:
         _LOGGER.debug("MsgProtocol.__init__(%s)", callback)
-        self._gwy = gwy  # is not used
+        # self._gwy = gwy  # is not used
         self._loop = gwy._loop
         self._callback = callback
 
