@@ -32,7 +32,6 @@ from .command import (
     QOS_TX_RETRIES,
     QOS_TX_TIMEOUT,
     Command,
-    Priority,
 )
 from .const import _PUZZ, HGI_DEVICE_ID, NON_DEVICE_ID, NUL_DEVICE_ID, __dev_mode__
 from .exceptions import InvalidPacketError
@@ -72,9 +71,6 @@ Pause = SimpleNamespace(
 
 VALID_CHARACTERS = printable  # "".join((ascii_letters, digits, ":-<*# "))
 
-INIT_QOS = {"priority": Priority.HIGHEST, "retries": 24, "disable_backoff": True}
-INIT_CMD = Command._puzzle(message=f"v{VERSION}", **INIT_QOS)
-
 # evofw3 commands (as of 0.7.0) include (from cmd.c):
 # case 'V':  validCmd = cmd_version( cmd );       break;
 # case 'T':  validCmd = cmd_trace( cmd );         break;
@@ -96,50 +92,14 @@ def _normalise(pkt_line: str, log_file: bool = False) -> str:
      - correct any historical design failures (e.g. puzzle packets with an index)
     """
 
-    # bug fixed in evofw3 v0.6.x...
-    # 095  I --- 18:013393 18:000730 --:------ 0001 005 00FFFF0200 # HGI80
-    # 000  I --- 18:140805 18:140805 --:------ 0001 005 00FFFF0200 # evofw3
-    if pkt_line[10:14] == " 18:" and pkt_line[11:20] == pkt_line[21:30]:
-        pkt_line = pkt_line[:21] + HGI_DEVICE_ID + pkt_line[30:]
-        (_LOGGER.warning if DEV_MODE else _LOGGER.debug)(
-            "evofw3 packet line has been normalised (0x00)"
-        )
-
     # psuedo-RAMSES-II packets...
-    elif pkt_line[10:14] in (" 08:", " 31:") and pkt_line[-16:] == "* Checksum error":
+    if pkt_line[10:14] in (" 08:", " 31:") and pkt_line[-16:] == "* Checksum error":
         pkt_line = pkt_line[:-17] + " # Checksum error (ignored)"
-        (_LOGGER.warning if DEV_MODE else _LOGGER.debug)(
+        (_LOGGER.error if DEV_MODE else _LOGGER.warning)(
             "Packet line has been normalised (0x01)"
         )
 
-    # bug fixed in evofw3 v0.6.x...
-    elif pkt_line[:2] == "!C":
-        pkt_line = "# " + pkt_line
-        (_LOGGER.warning if DEV_MODE else _LOGGER.debug)(
-            "Packet line has been normalised (0x02)"
-        )
-
-    # # TODO: very old evofw3 - taken out because expensive
-    # elif ERR_MSG_REGEX.match(pkt_line):
-    #     pkt_line = "# " + pkt_line
-    #     (_LOGGER.warning if DEV_MODE else _LOGGER.debug)(
-    #         "Packet line has been normalised (0x03)"
-    #     )
-
-    if not log_file:
-        return pkt_line
-
-    pkt_line = pkt_line.strip()
-
-    # HACK for v0.11.x and earlier - puzzle packets should have no index
-    if pkt_line[41:45] == _PUZZ and (pkt_line[:2] != "00" or pkt_line[:4] != "0000"):
-        payload = f"00{pkt_line[50:]}"[:96]
-        pkt_line = pkt_line[:46] + f"{int(len(payload)/2):03} " + payload
-        (_LOGGER.warning if DEV_MODE else _LOGGER.debug)(
-            "Packet line has been normalised (0x04)"
-        )
-
-    return pkt_line
+    return pkt_line.strip()
 
 
 def _str(value: ByteString) -> str:
@@ -217,16 +177,15 @@ class SerTransportPoll(asyncio.Transport):
                 self._protocol.data_received(self.serial.read(self.serial.in_waiting))
                 continue
 
-            # if hasattr(self.serial, "out_waiting") and self.serial.out_waiting:
-            # if getattr(self.serial, "out_waiting", False):
-            if self.serial and self.serial.out_waiting:
+            if getattr(self.serial, "out_waiting", False):
+                # NOTE: rfc2217 ports have no out_waiting attr!
                 continue
 
             if not self._write_queue.empty():
                 self.serial.write(self._write_queue.get())
                 self._write_queue.task_done()
 
-        self._protocol.connection_lost()
+        self._protocol.connection_lost(exc=None)
 
     def write(self, cmd):
         """Write some data bytes to the transport.
@@ -271,32 +230,25 @@ class _SerTransportProc(Process):  # TODO: WIP
 
     def _polling_loop(self):
         # asyncio.set_event_loop(self._loop)
-        asyncio.get_running_loop()  # TODO: this fails
+        # asyncio.get_running_loop()  # TODO: this fails
 
         self._protocol.connection_made(self)
 
         while self.serial.is_open:
+            # time.sleep(0.001)
+
             if self.serial.in_waiting:
-                self._protocol.data_received(
-                    # self.serial.readline()
-                    self.serial.read()
-                    # self.serial.read(self.serial.in_waiting)
-                )
-                # time.sleep(0.005)
+                # NOTE: cant use readline(), as it blocks until a newline is received
+                self._protocol.data_received(self.serial.read(self.serial.in_waiting))
                 continue
 
-            if self.serial.out_waiting:
-                # time.sleep(0.005)
+            if self.serial and getattr(self.serial, "out_waiting", False):
+                # NOTE: rfc2217 ports have no out_waiting attr!
                 continue
 
             if not self._write_queue.empty():
-                cmd = self._write_queue.get()
-                self.serial.write(bytearray(f"{cmd}\r\n".encode("ascii")))
+                self.serial.write(self._write_queue.get())
                 self._write_queue.task_done()
-                # time.sleep(0.005)
-                continue
-
-            # time.sleep(0.005)
 
         self._protocol.connection_lost(exc=None)
 
@@ -318,7 +270,9 @@ class PacketProtocolBase(asyncio.Protocol):
     to transport: self.send_data(cmd)       -> self._transport.write(bytes)
     """
 
-    _dt_now = dt_now if sys.platform == "win32" else dt.now
+    @staticmethod
+    def _dt_now():
+        return dt_now() if sys.platform == "win32" else dt.now()
 
     def __init__(self, gwy, pkt_handler: Callable) -> None:
         self._gwy = gwy
@@ -389,26 +343,33 @@ class PacketProtocolBase(asyncio.Protocol):
                 self._unwanted.append(dev_id)
                 return
 
-            if dev_id in (NON_DEVICE_ID, NUL_DEVICE_ID):
+            if dev_id in self._include or dev_id in (NON_DEVICE_ID, NUL_DEVICE_ID):
                 continue
 
-            if not self._include or dev_id in self._include:
-                continue  # check the other device_id, if any
-
             if dev_id == self._hgi80[DEVICE_ID]:
-                _LOGGER.warning(
-                    f"Allowing packets with device_id: {dev_id} (is active gateway?), "
-                    f"configure the {KNOWN_LIST}/{BLOCK_LIST} as required"
-                )
-                self._include.append(dev_id)  # the only time include list is modified
+                if self._include:
+                    _LOGGER.warning(
+                        f"Allowing packets with device_id: {dev_id} (is gateway), "
+                        f"configure the {KNOWN_LIST}/{BLOCK_LIST} as required"
+                    )
+                    self._include.append(dev_id)  # only time include list is modified
                 continue
 
-            _LOGGER.warning(
-                f"Blocking packets with device_id: {dev_id} (is not whitelisted), "
-                f"if required, add it to the {KNOWN_LIST}"
-            )
-            self._unwanted.append(dev_id)
-            return
+            if dev_id[:2] == "18" and self._gwy.serial_port:  # dex
+                _LOGGER.warning(
+                    f"Blocking packets with device_id: {dev_id} (is foreign gateway), "
+                    f"configure the {KNOWN_LIST}/{BLOCK_LIST} as required"
+                )
+                self._unwanted.append(dev_id)
+                return
+
+            if self._include:
+                _LOGGER.warning(
+                    f"Blocking packets with device_id: {dev_id} (is not whitelisted), "
+                    f"if required, add it to the {KNOWN_LIST}"
+                )
+                self._unwanted.append(dev_id)
+                return
 
         return True
 
@@ -445,7 +406,7 @@ class PacketProtocolBase(asyncio.Protocol):
                 _LOGGER.error("%s < Cant create packet (ignoring): %s", line, exc)
             return
 
-        if pkt.src.type == "18":
+        if pkt.src.type == "18":  # dex: should use HGI, but how?
             if self._hgi80[DEVICE_ID] is None:
                 self._hgi80[DEVICE_ID] = pkt.src.id
 
@@ -480,12 +441,12 @@ class PacketProtocolBase(asyncio.Protocol):
         while self._pause_writing:
             await asyncio.sleep(0.005)
 
-        while (
-            self._transport is None
-            # or self._transport.serial is None  # Shouldn't be required, but is!
-            or getattr(self._transport.serial, "out_waiting", False)
-        ):
-            await asyncio.sleep(0.005)
+        # while (
+        #     self._transport is None
+        #     # or self._transport.serial is None  # Shouldn't be required, but is!
+        #     or getattr(self._transport.serial, "out_waiting", False)
+        # ):
+        #     await asyncio.sleep(0.005)
 
         data = bytes(data.encode("ascii"))
 
@@ -519,7 +480,7 @@ class PacketProtocolBase(asyncio.Protocol):
                     cmd.src.id,
                     cmd.tx_header,
                 )
-            await self.send_data(Command._puzzle("02", cmd.tx_header))
+            await self.send_data(Command._puzzle(msg_type="11", message=cmd.tx_header))
 
         await self._send_data(str(cmd))
 
@@ -557,7 +518,7 @@ class PacketProtocolPort(PacketProtocolBase):
 
         # add this to start of the pkt log, if any
         if not self._disable_sending:
-            self._loop.create_task(self.send_data(INIT_CMD))
+            self._loop.create_task(self.send_data(Command._puzzle()))
 
         self.resume_writing()
 
@@ -633,7 +594,8 @@ class PacketProtocolQos(PacketProtocolPort):
         self._timeout_half = None
 
     def _timeouts(self, dtm: dt) -> Tuple[dt, dt]:
-        """Update self._timeout_full, self._timeout_half"""
+        """Update self._timeout_full, self._timeout_half attrs."""
+
         if self._qos_cmd:
             if self._tx_hdr:
                 timeout = QOS_TX_TIMEOUT
@@ -789,7 +751,7 @@ class PacketProtocolQos(PacketProtocolPort):
             else:
                 if self._qos_cmd.code != _PUZZ:  # HACK: why expired when shouldn't
                     _logger_send(
-                        _LOGGER.warning,
+                        (_LOGGER.warning if DEV_MODE else _LOGGER.debug),
                         f"EXPIRED ({self._tx_retries}/{self._tx_retry_limit})",
                     )
                     _expired_cmd(self._qos_cmd)
